@@ -261,6 +261,7 @@ typedef struct
   unsigned is_icon_shaped : 1; /* is the icon shaped? */
   unsigned is_icon_unmapped : 1; /* was the icon unmapped, even though the
 				  * window is still iconified (Transients) */
+  unsigned is_in_transient_subtree : 1; /* temporary flag used in stack.c */
   unsigned is_mapped : 1; /* is it mapped? */
   unsigned is_map_pending : 1; /* Sent an XMapWindow, but didn't receive a
 				* MapNotify yet.*/
@@ -271,6 +272,8 @@ typedef struct
   unsigned is_partially_visible : 1; /* is the window partially visible */
   unsigned is_pixmap_ours : 1; /* is the icon pixmap ours to free? */
   unsigned is_placed_wb3 : 1;
+  unsigned is_scheduled_for_destroy : 1; /* mark window to be destroyed after
+					  * last complex func has finished. */
   unsigned is_scheduled_for_raise : 1; /* mark window to be raised after
 					* function execution. */
   unsigned is_size_inc_set : 1;
@@ -408,6 +411,12 @@ typedef struct window_style
  */
 typedef struct FvwmWindow
 {
+  char *name;                 /* name of the window */
+  char *icon_name;            /* name of the icon */
+#ifdef I18N_MB
+  char **name_list;           /* window name list */
+  char **icon_name_list;      /* icon name list */
+#endif
   struct FvwmWindow *next;    /* next fvwm window */
   struct FvwmWindow *prev;    /* prev fvwm window */
   struct FvwmWindow *stack_next; /* next (lower) fvwm window in stacking
@@ -426,6 +435,7 @@ typedef struct FvwmWindow
   Window title_w;             /* the title bar window */
   Window sides[4];
   Window corners[4];          /* Corner pieces */
+  window_flags flags;
   int nr_left_buttons;
   int nr_right_buttons;
   Window button_w[NUMBER_OF_BUTTONS];
@@ -452,12 +462,6 @@ typedef struct FvwmWindow
   Pixmap iconPixmap;          /* pixmap for the icon */
   int iconDepth;              /* Drawable depth for the icon */
   Pixmap icon_maskPixmap;     /* pixmap for the icon mask */
-  char *name;                 /* name of the window */
-  char *icon_name;            /* name of the icon */
-#ifdef I18N_MB
-  char **name_list;           /* window name list */
-  char **icon_name_list;      /* icon name list */
-#endif
   FvwmFont title_font;
   FvwmFont icon_font;
   XWindowAttributes attr;     /* the child window attributes */
@@ -469,8 +473,6 @@ typedef struct FvwmWindow
   int DeIconifyDesk;          /* Desk to deiconify to, for StubbornIcons */
   Window transientfor;
 
-  window_flags flags;
-
 #ifdef MINI_ICONS
   char *mini_pixmap_file;
   Picture *mini_icon;
@@ -502,6 +504,8 @@ typedef struct FvwmWindow
   int max_window_height;
   int shade_anim_steps;
   unsigned char grabbed_buttons;
+
+  void *pscratch;             /* multi purpose scratch pointer */
 } FvwmWindow;
 
 /* include this down here because FvwmWindows must be defined when including
