@@ -31,7 +31,6 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include <sys/wait.h>
 
 /* To add timing info to debug output, #define this: */
 #ifdef FVWM_DEBUG_TIME
@@ -50,6 +49,10 @@
 #include "events.h"
 #include "focus.h"
 
+static unsigned int grab_count[GRAB_MAXVAL] = { 1, 1, 0, 0, 0, 0, 0 };
+#define GRAB_EVMASK (ButtonPressMask | ButtonReleaseMask | ButtonMotionMask | PointerMotionMask | EnterWindowMask | LeaveWindowMask)
+
+
 int GetTwoArguments(char *action, int *val1, int *val2, int *val1_unit,
 		    int *val2_unit)
 {
@@ -58,104 +61,117 @@ int GetTwoArguments(char *action, int *val1, int *val2, int *val1_unit,
   return GetTwoPercentArguments(action, val1, val2, val1_unit, val2_unit);
 }
 
-/***************************************************************************
- *
- * Wait for all mouse buttons to be released
- * This can ease some confusion on the part of the user sometimes
- *
- * Discard superflous button events during this wait period.
- *
- ***************************************************************************/
-void WaitForButtonsUp(Bool do_handle_expose)
-{
-  Bool AllUp = False;
-  XEvent JunkEvent;
-  unsigned int mask;
 
-  while(!AllUp)
-  {
-    XAllowEvents(dpy,ReplayPointer,CurrentTime);
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX,
-		  &JunkY, &mask);
-    if(!(mask&(DEFAULT_ALL_BUTTONS_MASK)))
-      AllUp = True;
-    /* handle expose events */
-    if (do_handle_expose && XPending(dpy) &&
-	XCheckMaskEvent(dpy, ExposureMask, &Event))
-    {
-      DispatchEvent(True);
-    }
-  }
-  XSync(dpy,0);
-  while(XCheckMaskEvent(dpy,
-			ButtonPressMask|ButtonReleaseMask|ButtonMotionMask,
-			&JunkEvent))
-  {
-    StashEventTime (&JunkEvent);
-    XAllowEvents(dpy,ReplayPointer,CurrentTime);
-  }
+/*****************************************************************************
+ * Change the appearance of the grabbed cursor.
+ ****************************************************************************/
+static void change_grab_cursor(int cursor)
+{
+  if (cursor != None)
+    XChangeActivePointerGrab(
+      dpy, GRAB_EVMASK, Scr.FvwmCursors[cursor], CurrentTime);
+
+  return;
 }
 
 /*****************************************************************************
- *
- * Grab the pointer and keyboard.
- * grab_context: GRAB_NORMAL, GRAB_BUSY, GRAB_MENU, GRAB_BUSYMENU.
+ * Grab the pointer.
+ * grab_context: GRAB_NORMAL, GRAB_BUSY, GRAB_MENU, GRAB_BUSYMENU,
+ * GRAB_PASSIVE.
  * GRAB_STARTUP and GRAB_NONE are used at startup but are not made
  * to be grab_context.
+ * GRAB_PASSIVE does not actually grab, but only delays the following ungrab
+ * until the GRAB_PASSIVE is released too.
  ****************************************************************************/
+#define DEBUG_GRAB 0
+#if DEBUG_GRAB
+void print_grab_stats(char *text)
+{
+  int i;
+
+  fprintf(stderr,"grab_stats (%s):", text);
+  for (i = 0; i < GRAB_MAXVAL; i++)
+    fprintf(stderr," %d", grab_count[i]);
+  fprintf(stderr," \n");
+
+  return;
+}
+#endif
 Bool GrabEm(int cursor, int grab_context)
 {
-  int i=0,val=0;
-  Window grab_win;
-  unsigned int mask;
+  int i = 0;
+  int val = 0;
   int rep;
+  Window grab_win;
 
-  /* menus.c control itself its busy stuff. No busy stuff at start up.
-   * Only one busy grab */
-  if ( ((GrabPointerState & (GRAB_MENU | GRAB_STARTUP))
-	                 && (grab_context & GRAB_BUSY)) ||
-       ((GrabPointerState & GRAB_BUSY) && (grab_context & GRAB_BUSY)) )
+  if (grab_context <= GRAB_STARTUP || grab_context >= GRAB_MAXVAL)
+  {
+    fvwm_msg(
+      ERR, "GrabEm", "Bug: Called with illegal context %d", grab_context);
     return False;
+  }
+
+  if (grab_context == GRAB_PASSIVE)
+  {
+    grab_count[grab_context]++;
+    grab_count[GRAB_ALL]++;
+    return True;
+  }
+
+  if (grab_count[GRAB_ALL] > grab_count[GRAB_PASSIVE])
+  {
+    /* already grabbed, just change the grab cursor */
+    grab_count[grab_context]++;
+    grab_count[GRAB_ALL]++;
+    if (grab_context != GRAB_BUSY || grab_count[GRAB_STARTUP] == 0)
+    {
+      change_grab_cursor(cursor);
+    }
+    return True;
+  }
 
-  XSync(dpy,0);
   /* move the keyboard focus prior to grabbing the pointer to
    * eliminate the enterNotify and exitNotify events that go
    * to the windows. But GRAB_BUSY. */
-  if (grab_context != GRAB_BUSY)
-  {
-    if (!Scr.Focus || do_accept_input_focus(Scr.Focus))
-    {
-      /* But do not grab if the window does not accept input focus from the
-       * window manager */
-      if(Scr.PreviousFocus == NULL)
-	Scr.PreviousFocus = Scr.Focus;
-      SetFocus(Scr.NoFocusWin,NULL,0);
-    }
-    grab_win = Scr.Root;
-    rep = 500;
-  }
-  else
+  switch (grab_context)
   {
+  case GRAB_BUSY:
     if ( Scr.Hilite != NULL )
       grab_win = Scr.Hilite->w;
     else
       grab_win = Scr.Root;
     /* retry to grab the busy cursor only once */
     rep = 2;
+    break;
+  case GRAB_PASSIVE:
+    /* cannot happen */
+    return False;
+  default:
+    if (!Scr.Focus || do_accept_input_focus(Scr.Focus))
+    {
+      /* But do not grab if the window does not accept input focus from the
+       * window manager */
+      if(Scr.PreviousFocus == NULL)
+	Scr.PreviousFocus = Scr.Focus;
+      DeleteFocus(0);
+    }
+    grab_win = Scr.Root;
+    rep = 500;
+    break;
   }
-  mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|PointerMotionMask
-    | EnterWindowMask | LeaveWindowMask;
+
+  XSync(dpy,0);
   while((i < rep)&&
 	(val = XGrabPointer(
-	  dpy, grab_win, True, mask, GrabModeAsync, GrabModeAsync, Scr.Root,
-	  Scr.FvwmCursors[cursor], CurrentTime) != GrabSuccess))
+	  dpy, grab_win, True, GRAB_EVMASK, GrabModeAsync, GrabModeAsync,
+	  Scr.Root, Scr.FvwmCursors[cursor], CurrentTime) != GrabSuccess))
   {
     switch (val)
     {
     case GrabInvalidTime:
     case GrabNotViewable:
       /* give up */
-      i += 100000;
+      i += rep;
       break;
     case GrabSuccess:
       break;
@@ -169,60 +185,106 @@ Bool GrabEm(int cursor, int grab_context)
       break;
     }
   }
+  XSync(dpy,0);
 
   /* If we fall out of the loop without grabbing the pointer, its
-     time to give up */
-  XSync(dpy,0);
-  if(val!=GrabSuccess)
+   * time to give up */
+  if (val != GrabSuccess)
   {
     return False;
   }
-  GrabPointerState |= grab_context;
+  grab_count[grab_context]++;
+  grab_count[GRAB_ALL]++;
+#if DEBUG_GRAB
+print_grab_stats("grabbed");
+#endif
   return True;
 }
 
 
 /*****************************************************************************
  *
- * UnGrab the pointer and keyboard
+ * UnGrab the pointer
  *
  ****************************************************************************/
 void UngrabEm(int ungrab_context)
 {
-  Window w;
-
-  /* menus.c control itself regarbing */
-  if ((GrabPointerState & GRAB_MENU) && !(ungrab_context & GRAB_MENU))
+  if (ungrab_context <= GRAB_ALL || ungrab_context >= GRAB_MAXVAL)
   {
-    GrabPointerState = (GrabPointerState & ~ungrab_context) | GRAB_BUSYMENU;
+    fvwm_msg(
+      ERR, "UngrabEm", "Bug: Called with illegal context %d", ungrab_context);
     return;
   }
-  /* see if we need to regrab */
-  if ((GrabPointerState & GRAB_BUSY) && !(ungrab_context & GRAB_BUSY))
+
+  if (grab_count[ungrab_context] == 0 || grab_count[GRAB_ALL] == 0)
   {
-    GrabPointerState &= ~ungrab_context & ~GRAB_BUSY;
-    GrabEm(CRS_WAIT, GRAB_BUSY);
+    /* context is not grabbed */
     return;
   }
 
   XSync(dpy,0);
-  XUngrabPointer(dpy,CurrentTime);
-
-  GrabPointerState &= ~ungrab_context;
-  if(Scr.PreviousFocus != NULL && !(GRAB_BUSY & ungrab_context))
+  grab_count[ungrab_context]--;
+  grab_count[GRAB_ALL]--;
+  if (grab_count[GRAB_ALL] > 0)
   {
-    w = Scr.PreviousFocus->w;
+    int new_cursor = None;
 
-    /* if the window still exists, focus on it */
-    if (w)
+    /* there are still grabs left - switch grab cursor */
+    switch (ungrab_context)
     {
-      SetFocus(w,Scr.PreviousFocus,0);
+    case GRAB_NORMAL:
+    case GRAB_BUSY:
+      if (grab_count[GRAB_BUSYMENU] > 0)
+	new_cursor = CRS_WAIT;
+      else if (grab_count[GRAB_BUSY] > 0)
+	new_cursor = CRS_WAIT;
+      else
+	new_cursor = None;
+      break;
+    case GRAB_MENU:
+      if (grab_count[GRAB_BUSY] > 0)
+	new_cursor = CRS_WAIT;
+      else if (grab_count[GRAB_BUSYMENU] > 0)
+	new_cursor = CRS_MENU;
+      else if (grab_count[GRAB_MENU] > 0)
+	new_cursor = CRS_MENU;
+      else
+	new_cursor = None;
+      break;
+    case GRAB_BUSYMENU:
+      /* switch back from busymenu cursor to normal menu cursor */
+      new_cursor = CRS_MENU;
+      break;
+    default:
+      new_cursor = None;
+      break;
+    }
+    if (grab_count[GRAB_ALL] > grab_count[GRAB_PASSIVE])
+    {
+#if DEBUG_GRAB
+print_grab_stats("-restore");
+#endif
+      change_grab_cursor(new_cursor);
+    }
+  }
+  else
+  {
+#if DEBUG_GRAB
+print_grab_stats("-ungrab");
+#endif
+    XUngrabPointer(dpy, CurrentTime);
+    if (Scr.PreviousFocus && ungrab_context != GRAB_BUSY)
+    {
+      /* if the window still exists, focus on it */
+      if (Scr.PreviousFocus->w)
+	SetFocusWindow(Scr.PreviousFocus, 0);
+      Scr.PreviousFocus = NULL;
     }
-    Scr.PreviousFocus = NULL;
   }
   XSync(dpy,0);
-}
 
+  return;
+}
 
 #ifndef fvwm_msg /* Some ports (i.e. VMS) define their own version */
 /*
@@ -335,18 +397,6 @@ void NewFontAndColor(Font newfont, Pixel color, Pixel backcolor)
   XChangeGC(dpy,Scr.TitleGC,Globalgcm,&Globalgcv);
 }
 
-void ReapChildren(void)
-{
-#if HAVE_WAITPID
-  while ((waitpid(-1, NULL, WNOHANG)) > 0)
-    ;
-#elif HAVE_WAIT3
-  while ((wait3(NULL, WNOHANG, NULL)) > 0)
-    ;
-#else
-# error One of waitpid or wait3 is needed.
-#endif
-}
 
 /****************************************************************************
  *
