@@ -488,11 +488,22 @@ static void InteractiveMove(
     origDragY = DragY;
   }
 
-  if(IS_MAPPED(tmp_win) && DragWidth*DragHeight <
-     (Scr.OpaqueSize*Scr.MyDisplayWidth*Scr.MyDisplayHeight)/100)
-    do_move_opaque = True;
-  else if (IS_ICONIFIED(tmp_win))
+  if (IS_ICONIFIED(tmp_win))
     do_move_opaque = True;
+  else if (IS_MAPPED(tmp_win))
+  {
+    float areapct;
+
+    areapct = 100.0;
+    areapct *= ((float)DragWidth / (float)Scr.MyDisplayWidth);
+    areapct *= ((float)DragHeight / (float)Scr.MyDisplayHeight);
+    /* round up */
+    areapct += 0.1;
+    if ((float)areapct <= (float)Scr.OpaqueSize)
+    {
+      do_move_opaque = True;
+    }
+  }
 
   if (do_move_opaque)
   {
@@ -1282,7 +1293,6 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
       done = True;
       break;
     case ButtonPress:
-      XAllowEvents(dpy, AsyncPointer, CurrentTime);
       if (Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS &&
 	  ((Button1Mask << (Event.xbutton.button - 1)) & button_mask))
       {
@@ -1717,7 +1727,7 @@ void SetXOR(F_CMD_ARGS)
   if (Scr.XorGC)
     XChangeGC(dpy, Scr.XorGC, gcm, &gcv);
   else
-    Scr.XorGC = XCreateGC(dpy, Scr.Root, gcm, &gcv);
+    Scr.XorGC = fvwmlib_XCreateGC(dpy, Scr.Root, gcm, &gcv);
 
   /* free up XorPixmap if neccesary */
   if (XorPixmap != None) {
@@ -1778,7 +1788,7 @@ void SetXORPixmap(F_CMD_ARGS)
   if (Scr.XorGC)
     XChangeGC(dpy, Scr.XorGC, gcm, &gcv);
   else
-    Scr.XorGC = XCreateGC(dpy, Scr.Root, gcm, &gcv);
+    Scr.XorGC = fvwmlib_XCreateGC(dpy, Scr.Root, gcm, &gcv);
 }
 
 
@@ -2165,7 +2175,6 @@ void resize_window(F_CMD_ARGS)
     switch(Event.type)
     {
     case ButtonPress:
-      XAllowEvents(dpy, AsyncPointer, CurrentTime);
       done = True;
       if (Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS &&
 	  ((Button1Mask << (Event.xbutton.button - 1)) & button_mask))
