@@ -64,6 +64,7 @@ typedef struct
 typedef enum
 {
     /* button types */
+    DefaultVectorButton         ,
     VectorButton                ,
     SimpleButton                ,
     GradientButton              ,
@@ -371,9 +372,11 @@ typedef struct ScreenInfo
     unsigned has_default_color_changed : 1;
     unsigned has_mouse_binding_changed : 1;
     unsigned has_nr_buttons_changed : 1;
+    unsigned is_executing_complex_function : 1;
     unsigned is_map_desk_in_progress : 1;
     unsigned is_pointer_on_this_screen : 1;
     unsigned is_single_screen : 1;
+    unsigned is_window_scheduled_for_destroy : 1;
     unsigned is_wire_frame_displayed : 1;
     unsigned silent_functions : 1;
     unsigned windows_captured : 1;
